@@ -65,7 +65,7 @@ class InstallKernelThread(threading.Thread):
         f.flush()
         comnd = subprocess.Popen(' '.join(cmd), stdout=self.application.logger.log, stderr=self.application.logger.log, shell=True)
         returnCode = comnd.wait()
-        f.close()   
+        f.close()
 
 class SidebarSwitcherRow(Gtk.ListBoxRow):
     def __init__(self, name, widget):
@@ -104,7 +104,7 @@ class SidebarSwitcher(Gtk.Bin):
         self.stack = stack
 
 class KernelRow(Gtk.ListBoxRow):
-    def __init__(self, version, pkg_version, text, installed, used, recommended, installable, window, application):
+    def __init__(self, version, pkg_version, text, installed, used, title, installable, window, application):
         Gtk.ListBoxRow.__init__(self)
 
         self.application = application
@@ -126,22 +126,14 @@ class KernelRow(Gtk.ListBoxRow):
         info_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
         info_box.set_spacing(6)
         hbox.pack_end(info_box, False, False, 0)
-        INFO_GROUP.add_widget(info_box) 
+        INFO_GROUP.add_widget(info_box)
 
-        if installed:
+        if title != "":
             label = Gtk.Label()
             label.set_margin_right(6)
             label.set_margin_left(6)
             label.props.xalign = 0.5
-            label.set_markup("<i>%s</i>" % _("Installed"))
-            Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(label), "dim-label")
-            hbox.pack_start(label, True, False, 0)
-        elif recommended:
-            label = Gtk.Label()
-            label.set_margin_right(6)
-            label.set_margin_left(6)
-            label.props.xalign = 0.5
-            label.set_markup("<i>%s</i>" % _("Recommended"))
+            label.set_markup("<i>%s</i>" % title)
             Gtk.StyleContext.add_class(Gtk.Widget.get_style_context(label), "dim-label")
             hbox.pack_start(label, True, False, 0)
 
@@ -163,7 +155,10 @@ class KernelRow(Gtk.ListBoxRow):
         link.set_line_wrap(True)
         box.pack_start(link, False, False, 2)
         link = Gtk.Label()
-        link.set_markup("<a href='http://changelogs.ubuntu.com/changelogs/pool/main/l/linux/linux_%s/changelog'>Changelog</a>" % pkg_version)
+        changelog_version = pkg_version
+        if "~" in pkg_version:
+            changelog_version = pkg_version.split("~")[0]
+        link.set_markup("<a href='http://changelogs.ubuntu.com/changelogs/pool/main/l/linux/linux_%s/changelog'>Changelog</a>" % changelog_version)
         link.set_line_wrap(True)
         box.pack_start(link, False, False, 2)
         link = Gtk.Label()
@@ -221,7 +216,7 @@ class KernelWindow():
         builder = Gtk.Builder()
         builder.add_from_file(gladefile)
         self.window = builder.get_object("window1")
-        self.window.set_title("Kernels")
+        self.window.set_title(_("Kernels"))
         listbox_series = builder.get_object("listbox_series")
         scrolled_series = builder.get_object("box7")
         kernel_stack_box = builder.get_object("box1")
@@ -236,7 +231,9 @@ class KernelWindow():
         # Setup the kernel warning page
         self.main_stack.add_named(info_box, "info_box")
         builder.get_object("button_continue").connect("clicked", self.on_continue_clicked, main_box)
+        builder.get_object("button_continue").set_label(_("Continue"))
         hide_info_checkbox = builder.get_object("checkbutton1").connect("toggled", self.on_info_checkbox_toggled)
+        builder.get_object("checkbutton1").set_label(_("Do not show this message again"))
 
         builder.get_object("title_warning").set_markup("<span font_weight='bold' size='x-large'>%s</span>" % _("Warning!"))
         builder.get_object("sub_title_warning").set_markup("<big><b>%s</b></big>" % _("Proceed with caution"))
@@ -255,26 +252,43 @@ class KernelWindow():
         kernel_stack_box.pack_start(stack, True, True, 0)
 
         builder.get_object("button_close").connect("clicked", self.hide_window)
+        builder.get_object("button_close").set_label(_("Close"))
 
-        kernels = subprocess.check_output("/usr/lib/linuxmint/mintUpdate/checkKernels.py | grep \"###\"", shell = True).decode("utf-8")
+        kernels = subprocess.check_output("/usr/lib/linuxmint/mintUpdate/checkKernels.py | sort | grep \"###\"", shell = True).decode("utf-8")
         kernels = kernels.split("\n")
         kernel_list = []
         pages_needed = []
         for kernel in kernels:
             values = kernel.split('###')
-            if len(values) == 7:
+            if len(values) == 9:
                 status = values[0]
                 if status != "KERNEL":
                     continue
-                (status, version, pkg_version, installed, used, recommended, installable) = values
+                (status, version_id, version, pkg_version, installed, used, recommended_stability, recommended_security, installable) = values
                 installed = (installed == "1")
                 used = (used == "1")
-                recommended = (recommended == "1")
+                title = ""
+                if used:
+                    title = _("Active")
+                elif installed:
+                    title = _("Installed")
+                recommend = None
+                if recommended_security == "1":
+                    recommend = _("Recommended for security")
+                elif recommended_stability == "1":
+                    recommend = _("Recommended for stability")
+
+                if recommend is not None:
+                    if title == "":
+                        title = recommend
+                    else:
+                        title = "%s - %s" % (recommend, title)
+
                 installable = (installable == "1")
                 label = version
 
                 page_label = label.split(".")[0] + "." + label.split(".")[1]
-                kernel_list.append([version, pkg_version, page_label, label, installed, used, recommended, installable])
+                kernel_list.append([version, pkg_version, page_label, label, installed, used, title, installable])
                 if page_label not in pages_needed:
                     pages_needed.append(page_label)
 
@@ -290,11 +304,11 @@ class KernelWindow():
             stack_switcher.add_titled(page, page)
 
             for kernel in kernel_list:
-                (version, pkg_version, page_label, label, installed, used, recommended, installable) = kernel
+                (version, pkg_version, page_label, label, installed, used, title, installable) = kernel
                 if used:
                     current_label.set_markup("<b>%s %s</b>" % (_("You are currently using the following kernel:"), kernel[3]))
                 if page_label == page:
-                    row = KernelRow(version, pkg_version, label, installed, used, recommended, installable, self.window, self.application)
+                    row = KernelRow(version, pkg_version, label, installed, used, title, installable, self.window, self.application)
                     list_box.add(row)
 
             list_box.connect("row_activated", self.on_row_activated)
